@@ -1,15 +1,21 @@
+//! # Cli module
+//! Cli module takes care of command line argument parsing and executing branches accordingly
+//!
+//! Cli module is only included in binary feature flag.
+
 use clap::clap_app;
-use crate::error::RadError;
+use crate::{error::RadError, processor::auth::AuthType};
 use crate::processor::Processor;
-use crate::logger::DebugOption;
+use crate::utils::Utils;
 use std::path::{Path, PathBuf};
 
 /// Struct to parse command line arguments and execute proper operations
 pub struct Cli{
     rules: Option<Vec<PathBuf>>,
-    debug_options: Option<Vec<DebugOption>>,
     write_to_file : Option<PathBuf>,
     error_to_file : Option<PathBuf>,
+    allow_auth: Option<Vec<AuthType>>,
+    allow_auth_warn: Option<Vec<AuthType>>,
 }
 
 impl Cli {
@@ -17,12 +23,16 @@ impl Cli {
     pub fn new() -> Self {
         Self {
             rules: None,
-            debug_options : None,
             write_to_file : None,
             error_to_file : None,
+            allow_auth : None,
+            allow_auth_warn : None,
         }
     }
 
+    /// User method to call cli workflow
+    ///
+    /// This sequentially parse command line arguments and execute necessary operations
     pub fn parse(&mut self) -> Result<(), RadError>{
         let cli_args = Cli::args_builder();
         self.run_processor(&cli_args)?;
@@ -33,20 +43,31 @@ impl Cli {
     fn run_processor(&mut self, args: & clap::ArgMatches) -> Result<(), RadError> {
 
         self.parse_options(args);
-
         // Build processor
         let mut processor = Processor::new()
             .purge(args.is_present("purge"))
             .greedy(args.is_present("greedy"))
             .strict(args.is_present("greedy"))
             .silent(args.is_present("silent"))
+            .allow(std::mem::replace(&mut self.allow_auth,None))?
+            .allow_with_warning(std::mem::replace(&mut self.allow_auth_warn,None))?
             .unix_new_line(args.is_present("newline"))
             .custom_rules(std::mem::replace(&mut self.rules,None))?
             .write_to_file(std::mem::replace(&mut self.write_to_file,None))?
+            .discard(args.is_present("discard"))?
             .error_to_file(std::mem::replace(&mut self.error_to_file,None))?
-            .debug(std::mem::replace(&mut self.debug_options,None))?
+            .debug(args.is_present("debug"))?
+            .log(args.is_present("log"))?
+            .interactive(args.is_present("interactive"))?
             .build();
 
+        // Debug
+        // Clear terminal cells
+        #[cfg(feature = "debug")]
+        if args.is_present("debug") {
+            Utils::clear_terminal()?;
+        }
+
         // ========
         // Main options
         // -->> Read from files
@@ -75,6 +96,7 @@ impl Cli {
         Ok(())
     }
 
+    /// Parse processor options
     fn parse_options(&mut self, args: & clap::ArgMatches) {
         // ========
         // Sub options
@@ -94,16 +116,45 @@ impl Cli {
             Some(PathBuf::from(error_file))
         } else { None };
 
-        self.debug_options = if let Some(debug_options) = args.value_of("debug") {
-            let option_vec = debug_options.split('+').map(|s| DebugOption::new(s)).collect::<Vec<DebugOption>>();
-            Some(option_vec)
+        // Permission
+        self.allow_auth = if let Some(auths) = args.value_of("allow") {
+            auths.split("+").map(|s| AuthType::from(s)).collect()
         } else { None };
+
+        // Permission with warning
+        self.allow_auth_warn = if let Some(auths) = args.value_of("allow_warn") {
+            auths.split("+").map(|s| AuthType::from(s)).collect()
+        } else { None };
+
+        // Permission all
+        if args.is_present("allow_all") {
+            self.allow_auth = Some(
+                vec![
+                    AuthType::FIN,
+                    AuthType::FOUT,
+                    AuthType::ENV,
+                    AuthType::CMD
+                ]
+            );
+        }
+
+        // Permission all with warning
+        if args.is_present("allow_all_warn") {
+            self.allow_auth_warn = Some(
+                vec![
+                    AuthType::FIN,
+                    AuthType::FOUT,
+                    AuthType::ENV,
+                    AuthType::CMD
+                ]
+            );
+        }
     }
 
-    /// Creates argument template
+    /// Creates argument template wich clap macro
     fn args_builder() -> clap::ArgMatches {
         clap_app!(rad =>
-            (version: "0.7.0")
+            (version: "0.8.0")
             (author: "Simon Creek <simoncreek@tutanota.com>")
             (about: "R4d is a modern macro processor made with rust")
             (@arg FILE: ... "Files to execute processing")
@@ -114,8 +165,15 @@ impl Cli {
             (@arg freeze: -f +takes_value "Freeze to file")
             (@arg purge: -p "Purge unused macros")
             (@arg strict: -S "Strict mode")
-            (@arg debug: -d --debug +takes_value "Debug mode [break|log|lines]")
+            (@arg debug: -d --debug "Debug mode")
+            (@arg log: -l --log "Debug log mode")
+            (@arg interactive: -i --interactive "Use interactive debug mode")
             (@arg combination: -c "Read from both stdin and file inputs")
+            (@arg discard: -D --discard "Discard output without prin out")
+            (@arg allow: -a +takes_value "Allow permission (io|cmd|env)")
+            (@arg allow_warn: -w +takes_value "Allow permission with warnings (io|cmd|env)")
+            (@arg allow_all: -A "Allow all permission")
+            (@arg allow_all_warn: -W "Allow all permission with warning")
             (@arg silent: -s "Supress error and warning")
             (@arg newline: -n "Use unix newline for formatting")
         ).get_matches()
